@@ -2,6 +2,7 @@ import logging
 
 import esphome.config_validation as cv
 from esphome.const import CONF_INPUT, CONF_MODE, CONF_NUMBER
+from esphome.pins import check_strapping_pin
 
 _ESP32P4_USB_JTAG_PINS = {24, 25}
 
@@ -13,13 +14,6 @@ _LOGGER = logging.getLogger(__name__)
 def esp32_p4_validate_gpio_pin(value):
     if value < 0 or value > 54:
         raise cv.Invalid(f"Invalid pin number: {value} (must be 0-54)")
-    if value in _ESP32P4_STRAPPING_PINS:
-        _LOGGER.warning(
-            "GPIO%d is a Strapping PIN and should be avoided.\n"
-            "Attaching external pullup/down resistors to strapping pins can cause unexpected failures.\n"
-            "See https://esphome.io/guides/faq.html#why-am-i-getting-a-warning-about-strapping-pins",
-            value,
-        )
     if value in _ESP32P4_USB_JTAG_PINS:
         _LOGGER.warning(
             "GPIO%d is reserved for the USB-Serial-JTAG interface.\n"
@@ -40,4 +34,5 @@ def esp32_p4_validate_supports(value):
     if is_input:
         # All ESP32 pins support input mode
         pass
+    check_strapping_pin(value, _ESP32P4_STRAPPING_PINS, _LOGGER)
     return value
