@@ -11,7 +11,7 @@ template<typename... Ts> class StartFanAction : public Action<Ts...> {
  public:
   explicit StartFanAction(SPS30Component *sps30) : sps30_(sps30) {}
 
-  void play(Ts... x) override { this->sps30_->start_fan_cleaning(); }
+  void play(const Ts &...x) override { this->sps30_->start_fan_cleaning(); }
 
  protected:
   SPS30Component *sps30_;
