@@ -2,13 +2,7 @@ import esphome.codegen as cg
 from esphome.components import sensor
 from esphome.const import CONF_ID, ICON_FLASH, UNIT_WATT_HOURS
 
-from .. import (
-    CONF_TAG_NAME,
-    TELEINFO_LISTENER_SCHEMA,
-    teleinfo_ns,
-    CONF_TELEINFO_ID,
-)
-
+from .. import CONF_TAG_NAME, CONF_TELEINFO_ID, TELEINFO_LISTENER_SCHEMA, teleinfo_ns
 
 TeleInfoSensor = teleinfo_ns.class_("TeleInfoSensor", sensor.Sensor, cg.Component)
 
