@@ -11,7 +11,7 @@ template<typename... Ts> class TemplateLockPublishAction : public Action<Ts...>,
  public:
   TEMPLATABLE_VALUE(lock::LockState, state)
 
-  void play(Ts... x) override { this->parent_->publish_state(this->state_.value(x...)); }
+  void play(const Ts &...x) override { this->parent_->publish_state(this->state_.value(x...)); }
 };
 
 }  // namespace template_
