@@ -11,7 +11,7 @@ template<typename... Ts> class TemplateValvePublishAction : public Action<Ts...>
   TEMPLATABLE_VALUE(float, position)
   TEMPLATABLE_VALUE(valve::ValveOperation, current_operation)
 
-  void play(Ts... x) override {
+  void play(const Ts &...x) override {
     if (this->position_.has_value())
       this->parent_->position = this->position_.value(x...);
     if (this->current_operation_.has_value())
