@@ -1,4 +1,6 @@
 #include "text.h"
+#include "esphome/core/defines.h"
+#include "esphome/core/controller_registry.h"
 #include "esphome/core/log.h"
 
 namespace esphome {
@@ -16,6 +18,9 @@ void Text::publish_state(const std::string &state) {
     ESP_LOGD(TAG, "'%s': Sending state %s", this->get_name().c_str(), state.c_str());
   }
   this->state_callback_.call(state);
+#if defined(USE_TEXT) && defined(USE_CONTROLLER_REGISTRY)
+  ControllerRegistry::notify_text_update(this);
+#endif
 }
 
 void Text::add_on_state_callback(std::function<void(std::string)> &&callback) {
