@@ -18,11 +18,10 @@ void UponorSmatrixComponent::setup() {
 
 void UponorSmatrixComponent::dump_config() {
   ESP_LOGCONFIG(TAG, "Uponor Smatrix");
-  ESP_LOGCONFIG(TAG, "  System address: 0x%04X", this->address_);
 #ifdef USE_TIME
   if (this->time_id_ != nullptr) {
     ESP_LOGCONFIG(TAG, "  Time synchronization: YES");
-    ESP_LOGCONFIG(TAG, "  Time master device address: 0x%04X", this->time_device_address_);
+    ESP_LOGCONFIG(TAG, "  Time master device address: 0x%08X", this->time_device_address_);
   }
 #endif
 
@@ -31,7 +30,7 @@ void UponorSmatrixComponent::dump_config() {
   if (!this->unknown_devices_.empty()) {
     ESP_LOGCONFIG(TAG, "  Detected unknown device addresses:");
     for (auto device_address : this->unknown_devices_) {
-      ESP_LOGCONFIG(TAG, "    0x%04X", device_address);
+      ESP_LOGCONFIG(TAG, "    0x%08X", device_address);
     }
   }
 }
@@ -89,8 +88,7 @@ bool UponorSmatrixComponent::parse_byte_(uint8_t byte) {
     return false;
   }
 
-  uint16_t system_address = encode_uint16(packet[0], packet[1]);
-  uint16_t device_address = encode_uint16(packet[2], packet[3]);
+  uint32_t device_address = encode_uint32(packet[0], packet[1], packet[2], packet[3]);
   uint16_t crc = encode_uint16(packet[packet_len - 1], packet[packet_len - 2]);
 
   uint16_t computed_crc = crc16(packet, packet_len - 2);
@@ -99,24 +97,14 @@ bool UponorSmatrixComponent::parse_byte_(uint8_t byte) {
     return false;
   }
 
-  ESP_LOGV(TAG, "Received packet: sys=%04X, dev=%04X, data=%s, crc=%04X", system_address, device_address,
+  ESP_LOGV(TAG, "Received packet: addr=%08X, data=%s, crc=%04X", device_address,
            format_hex(&packet[4], packet_len - 6).c_str(), crc);
 
-  // Detect or check system address
-  if (this->address_ == 0) {
-    ESP_LOGI(TAG, "Using detected system address 0x%04X", system_address);
-    this->address_ = system_address;
-  } else if (this->address_ != system_address) {
-    // This should never happen except if the system address was set or detected incorrectly, so warn the user.
-    ESP_LOGW(TAG, "Received packet from unknown system address 0x%04X", system_address);
-    return true;
-  }
-
   // Handle packet
   size_t data_len = (packet_len - 6) / 3;
   if (data_len == 0) {
     if (packet[4] == UPONOR_ID_REQUEST)
-      ESP_LOGVV(TAG, "Ignoring request packet for device 0x%04X", device_address);
+      ESP_LOGVV(TAG, "Ignoring request packet for device 0x%08X", device_address);
     return true;
   }
 
@@ -141,7 +129,7 @@ bool UponorSmatrixComponent::parse_byte_(uint8_t byte) {
       if (data[i].id == UPONOR_ID_DATETIME1)
         found_time = true;
       if (found_temperature && found_time) {
-        ESP_LOGI(TAG, "Using detected time device address 0x%04X", device_address);
+        ESP_LOGI(TAG, "Using detected time device address 0x%08X", device_address);
         this->time_device_address_ = device_address;
         break;
       }
@@ -160,7 +148,7 @@ bool UponorSmatrixComponent::parse_byte_(uint8_t byte) {
 
   // Log unknown device addresses
   if (!found && !this->unknown_devices_.count(device_address)) {
-    ESP_LOGI(TAG, "Received packet for unknown device address 0x%04X ", device_address);
+    ESP_LOGI(TAG, "Received packet for unknown device address 0x%08X ", device_address);
     this->unknown_devices_.insert(device_address);
   }
 
@@ -168,16 +156,16 @@ bool UponorSmatrixComponent::parse_byte_(uint8_t byte) {
   return true;
 }
 
-bool UponorSmatrixComponent::send(uint16_t device_address, const UponorSmatrixData *data, size_t data_len) {
-  if (this->address_ == 0 || device_address == 0 || data == nullptr || data_len == 0)
+bool UponorSmatrixComponent::send(uint32_t device_address, const UponorSmatrixData *data, size_t data_len) {
+  if (device_address == 0 || data == nullptr || data_len == 0)
     return false;
 
   // Assemble packet for send queue. All fields are big-endian except for the little-endian checksum.
   std::vector<uint8_t> packet;
   packet.reserve(6 + 3 * data_len);
 
-  packet.push_back(this->address_ >> 8);
-  packet.push_back(this->address_ >> 0);
+  packet.push_back(device_address >> 24);
+  packet.push_back(device_address >> 16);
   packet.push_back(device_address >> 8);
   packet.push_back(device_address >> 0);
 
