@@ -2,7 +2,7 @@ import esphome.codegen as cg
 from esphome.components import climate, remote_transmitter, sensor
 from esphome.components.remote_base import CONF_TRANSMITTER_ID
 import esphome.config_validation as cv
-from esphome.const import CONF_ID, CONF_SENSOR, CONF_SUPPORTS_COOL, CONF_SUPPORTS_HEAT
+from esphome.const import CONF_SENSOR, CONF_SUPPORTS_COOL, CONF_SUPPORTS_HEAT
 
 AUTO_LOAD = ["sensor"]
 
@@ -10,9 +10,9 @@ yashima_ns = cg.esphome_ns.namespace("yashima")
 YashimaClimate = yashima_ns.class_("YashimaClimate", climate.Climate, cg.Component)
 
 CONFIG_SCHEMA = cv.All(
-    climate.CLIMATE_SCHEMA.extend(
+    climate.climate_schema(YashimaClimate)
+    .extend(
         {
-            cv.GenerateID(): cv.declare_id(YashimaClimate),
             cv.GenerateID(CONF_TRANSMITTER_ID): cv.use_id(
                 remote_transmitter.RemoteTransmitterComponent
             ),
@@ -20,14 +20,14 @@ CONFIG_SCHEMA = cv.All(
             cv.Optional(CONF_SUPPORTS_HEAT, default=True): cv.boolean,
             cv.Optional(CONF_SENSOR): cv.use_id(sensor.Sensor),
         }
-    ).extend(cv.COMPONENT_SCHEMA)
+    )
+    .extend(cv.COMPONENT_SCHEMA)
 )
 
 
 async def to_code(config):
-    var = cg.new_Pvariable(config[CONF_ID])
+    var = await climate.new_climate(config)
     await cg.register_component(var, config)
-    await climate.register_climate(var, config)
 
     cg.add(var.set_supports_cool(config[CONF_SUPPORTS_COOL]))
     cg.add(var.set_supports_heat(config[CONF_SUPPORTS_HEAT]))
