@@ -26,10 +26,10 @@ class ZephyrGPIOPin : public InternalGPIOPin {
  protected:
   void attach_interrupt(void (*func)(void *), void *arg, gpio::InterruptType type) const override;
   uint8_t pin_;
-  bool inverted_;
-  gpio::Flags flags_;
-  const device *gpio_ = nullptr;
-  bool value_ = false;
+  bool inverted_{};
+  gpio::Flags flags_{};
+  const device *gpio_{nullptr};
+  bool value_{false};
 };
 
 }  // namespace zephyr
